{
  This demo shows how to copy data from/to table cell in RVF format.

  The main problem: cells do not support saving and loading RVF documents
  containing collections of text and paragraph styles.

  Workaround: using hidden TRichView.
    Copying from cell:
      Cell --[RVF without styles]--> hidden rv --[RVF with styles]--> ...
      (hidden rv is linked to the same RVStyle as the trichview containing the cell)

    Copying to cell
      ... --[RVF with styles]--> hidden rv --[RVF without styles]--> Cell
      (hidden rv is linked to the same RVStyle as the trichview containing the cell)

  You can copy SaveCellToRVF and LoadRVFInCell procedures in your project.
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, RVTable, RVEdit, RVScroll, RichView, RVStyle, CRVFData, StdCtrls;

type
  TForm1 = class(TForm)
    RVStyle1: TRVStyle;
    rv: TRichView;
    rveSource: TRichViewEdit;
    rveDest: TRichViewEdit;
    RVStyle2: TRVStyle;
    Button1: TButton;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
    procedure GenerateDoc;
    function SaveCellToRVF(Cell: TCustomRVFormattedData;
      rveSource, rv: TCustomRichView): TMemoryStream;
    procedure LoadRVFInCell(Stream: TStream; Cell: TCustomRVFormattedData;
      rveDest, rv: TCustomRichView);
    function FindFirstCell: TCustomRVFormattedData;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  GenerateDoc;
end;

// Creating a sample doc
procedure TForm1.GenerateDoc;
var table: TRVTableItemInfo;
begin
  rveSource.Clear;
  table := TRVTableItemInfo.CreateEx(1, 1, rveSource.RVData);
  table.CellBorderWidth := 1;
  table.BorderWidth := 2;
  table.BestWidth := -80;
  table.Cells[0,0].Clear;
  table.Cells[0,0].AddNL('Example', 1, 1);
  table.Cells[0,0].AddNL('This text will be copied as ', 0, 0);
  table.Cells[0,0].AddNL('RVF', 3, -1);
  table.ParaNo := 1;
  rveSource.AddItem('', table);
  rveSource.Format;
end;

// Saving data from cell with styles:
// 1. Saving from cell without styles
// 2. Loading in rv (linked to the same RVStyle)
// 3. Saving from rv with styles
function TForm1.SaveCellToRVF(Cell: TCustomRVFormattedData;
  rveSource, rv: TCustomRichView): TMemoryStream;
var OldRVFOptions: TRVFOptions;
begin
  // assigning properties of hidden TRichView
  // - should not read styles from rvf
  rv.RVFTextStylesReadMode := rvf_sIgnore;
  rv.RVFParaStylesReadMode := rvf_sIgnore;
  // - should save styles in rvf
  rv.RVFOptions := rv.RVFOptions+[rvfoSaveTextStyles,rvfoSaveParaStyles];
  // - should be linked to the same RVStyle
  rv.Style := rveSource.Style;

  // copying
  Result := TMemoryStream.Create;
  try
    OldRVFOptions := rveSource.RVFOptions;
    rveSource.RVFOptions := rveSource.RVFOptions-[rvfoSaveTextStyles,rvfoSaveParaStyles];
    try
      Cell.SaveRVFToStream(Result, False, clNone, nil, nil);
      Result.Position := 0;
      rv.LoadRVFFromStream(Result);
      Result.Clear;
      rv.SaveRVFToStream(Result, False);
      Result.Position := 0;
      rv.Clear;
    finally
      rveSource.RVFOptions := OldRVFOptions;
    end;
  except
    Result.Free;
    Result := nil;
  end;
end;

// Loading data in cell with styles:
// 1. Loading with styles to rv
// 2. Saving without styles
// 3. Loading in cell in rveStyles, linked to the same RVStyle as rv
procedure TForm1.LoadRVFInCell(Stream: TStream;
  Cell: TCustomRVFormattedData; rveDest, rv: TCustomRichView);
var Stream2: TMemoryStream;
    Color: TColor;
begin
  // assigning properties of hidden TRichView
  // - should read styles from rvf
  rv.RVFTextStylesReadMode := rvf_sInsertMerge;
  rv.RVFParaStylesReadMode := rvf_sInsertMerge;
  // - should not save styles in rvf
  rv.RVFOptions := rv.RVFOptions-[rvfoSaveTextStyles,rvfoSaveParaStyles];
  // - should be linked to the same RVStyle
  rv.Style := rveDest.Style;

  // copying
  Stream.Position := 0;
  rv.LoadRVFFromStream(Stream);
  Stream2 := TMemoryStream.Create;
  try
    rv.SaveRVFToStream(Stream2, False);
    Stream2.Position := 0;
    Cell.LoadRVFFromStream(Stream2, Color, nil, nil);
    rv.Clear;
  finally
    Stream2.Free;
  end;

end;

// Searching the first table cell in rvSource
function TForm1.FindFirstCell: TCustomRVFormattedData;
var i: Integer;
begin
  Result := nil;
  for i := 0 to rveSource.ItemCount-1 do
    if rveSource.GetItemStyle(i)=rvsTable then
      Result := TRVTableItemInfo(rveSource.GetItem(i)).Cells[0,0];
end;

// copying from the first cell of rveSource to rveDest
procedure TForm1.Button1Click(Sender: TObject);
var Stream: TStream;
    Cell: TCustomRVFormattedData;
begin
  // clearing the destination
  rveDest.Clear;
  rveDest.DeleteUnusedStyles(True, True, True);
  rveDest.Format;
  // searching for the cell to copy
  Cell := FindFirstCell;
  if Cell=nil then begin
    ShowMessage('No cell to copy');
    exit;
  end;
  // copying
  Stream := SaveCellToRVF(TCustomRVFormattedData(Cell.GetRVData), rveSource, rv);
  rveDest.LoadRVFFromStream(Stream);
  Stream.Free;
  rveDest.Format;
end;

// copying from the first cell of rveSource to rveDest as a table
procedure TForm1.Button2Click(Sender: TObject);
var Stream: TStream;
    Cell: TCustomRVFormattedData;
    table: TRVTableItemInfo;
begin
  // clearing the destination
  rveDest.Clear;
  rveDest.DeleteUnusedStyles(True, True, True);
  rveDest.Format;
  // searching for the cell to copy
  Cell := FindFirstCell;
  if Cell=nil then begin
    ShowMessage('No cell to copy');
    exit;
  end;
  // copying
  Stream := SaveCellToRVF(TCustomRVFormattedData(Cell.GetRVData), rveSource, rv);
  table := TRVTableItemInfo.CreateEx(1, 1, rveSource.RVData);
  table.CellBorderWidth := 3;
  table.Color := clMoneyGreen;
  table.CellBorderStyle := rvtbColor;
  table.BestWidth := -90;
  LoadRVFInCell(Stream, table.Cells[0,0], rveDest, rv);
  Stream.Free;
  rveDest.Clear;
  rveDest.AddItem('', table);
  rveDest.Format;
end;

end.
